module Board where

import Prelude hiding (Left, Right)
import Ix
import Array(Array, array, (//))

data Dir = Up | Down | Left | Right deriving (Eq, Ord, Enum, Show)

allDirs :: [Dir]
allDirs = [Up .. Right]


opposite :: Maybe Dir -> Maybe Dir -> Bool
opposite (Just Up) (Just Down) = True
opposite (Just Down) (Just Up) = True
opposite (Just Left) (Just Right) = True
opposite (Just Right) (Just Left) = True
opposite _ _ = False


-- A graph in our sense is something where from a given node, 
-- in a given direction, we may get to another node (or not)
type Graph a = a -> Dir-> Maybe a
type Maze = Graph Dot

-- A 'dot' is a node on our game graph (given by an Int)
type Dot = Int

-- A dot can either contain nothing, or a wee pill) or a big (power) pill:
data DotState = MtDot | WeePill | BigPill

-- The maze the game runs on:
maze :: Maze
maze 1 Right = Just 2
maze 1 Down  = Just 6
maze 2 Left  = Just 1
maze 2 Right = Just 3
maze 3 Left  = Just 2
maze 3 Right = Just 4
maze 3 Down  = Just 8
maze 4 Left  = Just 3
maze 4 Right = Just 5
maze 5 Left  = Just 4
maze 5 Down  = Just 10
maze 6 Right = Just 7
maze 6 Up    = Just 1
maze 7 Left  = Just 6
maze 7 Right = Just 8
maze 8 Left  = Just 7
maze 8 Right = Just 9
maze 8 Up    = Just 3
maze 9 Left  = Just 8
maze 9 Right = Just 10
maze 10 Left = Just 9
maze 10 Up   = Just 5

maze 7  Down = Just 12
maze 12 Up   = Just 7
maze 9  Down = Just 14
maze 14 Up   = Just 9

maze 11 Right = Just 12
maze 11 Down  = Just 16
maze 12 Left  = Just 11
maze 12 Right = Just 13
maze 13 Left  = Just 12
maze 13 Right = Just 14
maze 13 Down  = Just 18
maze 14 Left  = Just 13
maze 14 Right = Just 15
maze 15 Left  = Just 14
maze 15 Down  = Just 20
maze 16 Right = Just 17
maze 16 Up    = Just 11
maze 17 Left  = Just 16
maze 17 Right = Just 18
maze 18 Left  = Just 17
maze 18 Right = Just 19
maze 18 Up    = Just 13
maze 19 Left  = Just 18
maze 19 Right = Just 20
maze 20 Left  = Just 19
maze 20 Up    = Just 15
maze _  _     = Nothing

-- The maze size
mazeRows :: Int
mazeRows = 4
mazeCols :: Int
mazeCols = 5

-- All dots
allDots :: [Dot]
allDots = [1 .. 20]

-- Position of a single dot (as row/column)
pos :: Dot-> (Int, Int) 
pos  i = ((i- 1) `div` mazeCols, (i- 1) `mod` mazeCols)
                   

-- Initial state of the maze
initialmaze :: Array Dot DotState
initialmaze = array (1, length allDots) (map (\d-> (d, WeePill)) allDots)
               // [(2, BigPill), (4, BigPill), (17, BigPill), (19, BigPill)]

-- Where do we start?
playerStart :: Dot
playerStart = 13

