module Main where

import Prelude hiding (Left, Right)
import Maybe (listToMaybe, mapMaybe)
import Monad (when)

import Graphics.HGL
import State
import Board
import Drawing


forever :: IO a-> IO a
forever a = a >> forever a

waitForSpace :: Window-> IO ()
waitForSpace w = 
  do e<- getWindowEvent w
     case e of (Key {keysym= k, isDown= True}) | keyToChar k == ' ' -> return ()
               _ -> waitForSpace w

keyToDir :: Event-> Maybe Dir
keyToDir (Key {keysym= k, isDown= True})
     | isUpKey k    = Just Up
     | isDownKey k  = Just Down
     | isLeftKey k  = Just Left
     | isRightKey k = Just Right
keyToDir _  = Nothing


-- Get all events which have occured, in reverse order
getEvents :: Window-> [Event]-> IO [Event]
getEvents w evs =
  do x<- maybeGetWindowEvent w
     case x of
       Nothing -> return evs
       Just e  -> getEvents w (e: evs)


-- Run one project 
run1 :: Window-> State-> IO State
run1 w s = 
  do setGraphic w (draw s)
     getWindowTick w
     evs <- getEvents w []
     s'  <- nextState s (listToMaybe (mapMaybe keyToDir evs))
     if dead s then return s'
               else run1 w s'

-- Run whole game
run :: Window-> State-> IO Int
run w s = 
  do run1 w s
     if (lives s) == 0 then do
         waitForSpace w
         return (score s)
       else do
         waitForSpace w
         run w (newGame s)		      

-- Main function
main :: IO ()
main = runGraphics $
  do w<- openWindowEx "hsProman: The Haskell Project Simulator" Nothing
           (winX, winY) DoubleBuffered (Just 30)
     run1 w initialState
     closeWindow w

